
; ATmega48, ATmega88

; Secure-Bits fr Mega8:
; gesetzt SUT0, CKOPT 

; 10.05.2007 M'Soft Berlin
; 27.10.2011 de0508, Uwe: Anpassungen an den atMegax8

.equ	VER		= '0'
.equ	REV		= '6'

;.include "m8def.inc"
.include "m48def.inc"
;.include "m88def.inc"

// Anpassungen atMegax8 zurck an atMega8
.include "megax8.inc"

.include "Definitionen.inc"

;****************************** Verktor-Liste ************************

.CSEG
       rjmp    Reset


.ORG	OC2Aaddr
		rjmp	irq_1ms

.ORG	ICP1addr
		rjmp	irq_capture

.ORG	OVF0addr
		rjmp	irq_overflow
				 
;****************************** Programm-Beginn **********************

;*********************************************************************
;  Initialisierungsroutine
;*********************************************************************

.ORG	INT_VECTORS_SIZE

; Hardware initialisieren

Reset:
	ldi     TEMP_A,low(RAMEND)          ; Stack an das interne Ram-Ende
        out     SPL,TEMP_A
	ldi     TEMP_A,high(RAMEND)
        out     SPH,TEMP_A

; Port B *************************************************************

; Digit-Treiber			
	clr	TEMP_A							; alle Digits aus			
        out     PORTB,TEMP_A                	
        ldi     TEMP_A,(1<<DIG_1)|(1<<DIG_2)|(1<<DIG_3)|(1<<DIG_4)|(1<<SEG_DP)
	out     DDRB,TEMP_A						; Digits Ausgang

; Port C *************************************************************

; ADC			
	ldi	TEMP_A,(1<<KEY_PIN)				; PullUp fr Taste ein, Ausgnge L			
        out     PORTC,TEMP_A 
	ldi	TEMP_A,(1<<CLK_OUT)|(1<<BER_RC)|(1<<BER_R_IN)	; RC an GND, R_IN an GND
	out     DDRC,TEMP_A

; Port D *************************************************************

; Segment-Treiber
	clr		TEMP_A			
        out     PORTD,TEMP_A                	; Alle Segmente aus
        ldi     TEMP_A,(1<<SEG_A)|(1<<SEG_B)|(1<<SEG_C)|(1<<SEG_D)|(1<<SEG_E)|(1<<SEG_F)|(1<<SEG_G)
        out     DDRD,TEMP_A						; Segmente Ausgang

; TIMER 2 - 1ms (8Bit) ***********************************************

;	Vorteiler 64, Timer 250

.ifdef TCCR2B
	ldi	TEMP_A,low(1<<WGM21)	    ; CTC 
	STORE	TCCR2A,TEMP_A

	ldi	TEMP_A,low(1<<CS22)	    ; Prescale 64
	STORE	TCCR2B,TEMP_A
.else
	ldi	TEMP_A,(1<<WGM21)|(1<<CS22)    ; CTC, Prescale 64
	out	TCCR2,TEMP_A
.endif

        ldi     TEMP_A,TCONST_TIME				; Timer-Wert
	STORE	OCR2A,TEMP_A
	clr	TEMP_A
	STORE	ASSR,TEMP_A
	ldi	TEMP_A,(1<<OCIE2A)				; Timer-IRQ frei
        STORE	TIMSK2,TEMP_A  

; TIMER 1 - Counter (16Bit) ******************************************

	clr	TEMP_A				; Timer als Counter
	STORE	TCCR1A,TEMP_A
	clr	TEMP_A				; Counter stop
	STORE	TCCR1B,TEMP_A
        STORE     TCNT1H,NULL			; Counter auf 0	-> Grundstellung in allen Funktionen am Ende wiederherstellen!	
        STORE     TCNT1L,NULL		

; TIMER 0 - extern Clock (8Bit) **************************************

	ldi	TEMP_A,(1<<CS00)			    ; Prescale 0
	STORE	TCCR0B,TEMP_A
        ldi     TEMP_A,255-(F_CPU/(OUT_CLOCK*2000)); Timer-Wert
        STORE   TCNT0,TEMP_A		

; ADC ****************************************************************

		ldi		TEMP_A,(1<<REFS0)|(1<<REFS1)	; interne Referenz
		STORE		ADMUX,TEMP_A
; atMega8 undef: ADFR
;		ldi		TEMP_A,(1<<ADEN)|(1<<ADSC)|(1<<ADFR)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0) ; Free Running und starten
; atMega88
		ldi		TEMP_A,(1<<ADEN)|(1<<ADSC)|(1<<ADATE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0) ; Free Running und starten
		STORE		ADCSRA,TEMP_A
.ifdef ADCSRB
; 20111027 -us
		ldi		TEMP_A,0
		STORE		ADCSRB,TEMP_A
; ------------
.endif

;*********************************************************************				 

; Software initialisieren

	ldi	FLAGS,(1<<FLAG_U)				; Start mit Funktion Spannungsmesser
	clr	FLAGS_IRQ
	clr	NULL
	clr	DIG_DP							; alle Dezimalpunkte aus
	ldi	DIG_NR,(1<<DIG_1)				; Bit fr aktuelle Digitposition Digit 1
	ldi	DIG_ADR,low(DIGIT_1)			; Adresse Daten fr Digit 1

	STORE	DIGIT_1,NULL					; Alle Segmente aus
	STORE	DIGIT_2,NULL
	STORE	DIGIT_3,NULL
	STORE	DIGIT_4,NULL

	sei

	rcall	init							; ;Mit Ub und setzt Spannungs- und Logikparameter
					 
;*********************************************************************
;  Main-Loop
;*********************************************************************

main:	
		sbrs	FLAGS,FLAG_U			; Spannung messen?
		rjmp	main_freq				; nein

		rcall	set_10v

main_span:
		rcall	get_spannung
		ldi		TEMP_D,100				; Warteschleife, 100ms -> 10 Messungen/s
		rjmp	main_wait_loop

main_freq:
		sbrs	FLAGS,FLAG_F			; Frequenz messen
		rjmp	main_logik				; nein

		rcall	get_frequenz
		ldi		TEMP_D,100				; Warteschleife, 100ms -> 10 Messungen/s
		rjmp	main_wait_loop

main_logik:
		sbrs	FLAGS,FLAG_L			; Logikpegel
		rjmp	main_clock				; nein

		rcall	get_logik
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_logik				; nein

		rjmp	main_wait_key

main_clock:
		rcall	get_clock
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_clock				; nein

		rjmp	main_wait_key

main_wait_loop:
		sbrs	FLAGS,FLAG_IRQ			; auf IRQ warten
		rjmp	main_wait_loop

		cbr		FLAGS,(1<<FLAG_IRQ)		; Flag lschen fr nchsten IRQ

		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_wait_no_key		; nein

main_wait_key:
		rcall	key_decode				; Taste auswerten, wechselt Funktion U->F->L->C->U usw.
		rjmp	main_key_end

main_wait_no_key:
		dec		TEMP_D					; Warteschleife bei U und F
		brne	main_wait_loop

main_key_end:
		sbrc	FLAGS,FLAG_U			; Spannung messen
		rjmp	main_span

		sbrc	FLAGS,FLAG_F			; Frequenz messen
		rjmp	main_freq

		sbrc	FLAGS,FLAG_L			; Logik messen
		rjmp	main_logik

		sbrc	FLAGS,FLAG_C			; Clock ausgeben
		rjmp	main_clock

		rjmp	main					; hier sollten wir nie ankommen... ;)

;*********************************************************************				 

.include "Funktionen.inc"
.include "Subroutinen.inc"
.include "Irq.inc"
.include "Tabellen.inc"

